﻿''' <summary>オセロのセル(升目)を表します。セルの状態(黒・白・なし)の管理や、セルへのフォーカスの移動、グラフィックの描画を行います</summary>
Public Class Cell

    '変数の宣言

    ''' <summary>セルの状態 (なし・黒石・白石)</summary>
    Public Status As CellStatus

    ''' <summary>GridBoardのインスタンス。オセロのグリッド(盤面)を表します。石を置くためのメソッドや、ひっくり返すメソッドとそれに関連するイベント、セルの管理、描画処理を行います</summary>
    Public Board As GridBoard

    ''' <summary>セルの座標 (X,Y)</summary>
    Public CellPosition As Point

    ''' <summary>セルの位置 (石を置く四角)</summary>
    Public CellRectangle As Rectangle

    ''' <summary>セルに石を描画するためのインスタンス。とりあえず (0,0,0,0)</summary>
    Private CellBrushes As New SolidBrush(Color.FromArgb(0, 0, 0, 0))

    ''' <summary>セルにフォーカスがあるか</summary>
    Public Focused As Boolean

    ''' <summary>セルにおけるかどうか</summary>
    Public Puttabled As Boolean

    ''' <summary>ヒント表示フラグ</summary>
    Public Hint As Boolean


    ''' <summary>セルクラスのコンストラクタ。セルの座標を下にセルの位置を計算</summary>
    ''' <param name="Grid">セルがあるグリッドを指定</param>
    ''' <param name="Position">セルの座標を指定</param>
    Public Sub New(ByVal Grid As GridBoard, ByVal Position As Point)

        Me.Board = Grid
        Me.CellPosition = Position

        Dim CellRect As New Rectangle

        'セルの座標を下にセルの位置を計算
        CellRect.X = Position.X * GridBoard.CellSize    'X座標を指定
        CellRect.Y = Position.Y * GridBoard.CellSize    'Y座標を指定
        CellRect.Width = GridBoard.CellSize             '幅を指定
        CellRect.Height = GridBoard.CellSize            '高さを指定

        Me.CellRectangle = CellRect

    End Sub

    ''' <summary>現在のセルの状態より、セルに石を描画 (なし・黒石・白石)</summary>
    ''' <param name="g">グラフィックオブジェクト</param>
    Public Sub Draw(ByVal g As Graphics)

        Dim CellRect As New Rectangle

        CellRect = Me.CellRectangle
        CellRect.Inflate(-2, -2)


        'PuttableがTrueなら少し色を濃ゆくする
        If Me.Puttabled Then
            If Me.Hint Then
                'Hintフラグが有効ならそっち優先
                g.FillRectangle(Brushes.LightCoral, CellRect.X, CellRect.Y, CellRect.Width + 1, CellRect.Height + 1)
            Else
                g.FillRectangle(Brushes.LimeGreen, CellRect.X, CellRect.Y, CellRect.Width + 1, CellRect.Height + 1)
            End If
        End If

        '石の描画
        Select Case Me.Status
            Case CellStatus.Nothing
                'セルの状態が「なし」なら何もしない
            Case CellStatus.Black
                'セルの状態が「黒石」なら黒石を描画
                CellBrushes.Color = Color.Black
                g.FillEllipse(CellBrushes, CellRect)
            Case CellStatus.White
                'セルの状態が「白石」なら白石を描画
                CellBrushes.Color = Color.White
                g.FillEllipse(CellBrushes, CellRect)
        End Select

        'フォーカスがTrueなら枠を描画
        If Me.Focused Then
            Using LinePen As New Pen(Color.DarkOrange, 2)
                g.DrawRectangle(LinePen, CellRect.X, CellRect.Y, CellRect.Width + 1, CellRect.Height + 1)
            End Using
        End If

    End Sub

    ''' <summary>フォーカスを受け取ると、セルに枠を描画</summary>
    Public Sub Focus()

        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        '自分以外のセルのフォーカスを解除
        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                Board.Cells(X, Y).Focused = False
            Next
        Next

        '自分にフォーカスを当てる
        Me.Focused = True

    End Sub

    '■Puttable
    ''' <summary>セルにヒントを表示します</summary>
    ''' <remarks>アクティブなセルとはPuttabledプロパティがTrueのセルです
    ''' このメソッドを呼び出すと盤上の全てのセルをチェックし、置けるセルのPuttabledプロパティをTrueにします。
    ''' </remarks>
    Public Sub Puttable(ByVal PlayerColor As CellStatus)

        Dim X As Integer
        Dim Y As Integer

        'すべてのグリッドを調査
        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                Board.Cells(X, Y).Puttabled = False
                If Board.CanPut(PlayerColor, X, Y) Then
                    Board.Cells(X, Y).Puttabled = True
                End If
            Next
        Next

    End Sub

End Class
